﻿Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports System.Collections.ObjectModel
Imports ThinkGeo.MapSuite.Core
Imports ThinkGeo.MapSuite.DesktopEdition


Class RulerTrackInteractiveOverlay
    Inherits TrackInteractiveOverlay
    Private Const currentFeatureKey As String = "CurrentFeature"
    Private rulerLineShape As LineShape
    Private mouseDown As Integer

    Public Sub New()
        MyBase.New()
        TrackShapeLayer.Open()
        TrackShapeLayer.Columns.Add(New FeatureSourceColumn("length"))
        'Sets the appearance of the ruler
        TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = LineStyles.CreateSimpleLineStyle(GeoColor.FromArgb(150, GeoColor.StandardColors.DarkRed), 4, False)
        TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = New TextStyle("length", New GeoFont("Arial", 10, DrawingFontStyles.Bold), New GeoSolidBrush(GeoColor.SimpleColors.Black))
        TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.TextLineSegmentRatio = 100
        TrackShapeLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.YOffsetInPixel = 10
        TrackShapeLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20
    End Sub

    Protected Overrides Function MouseDownCore(interactionArguments As InteractionArguments) As InteractiveResult
        'Sets the first and last point of the line where the user clicks.
        Dim interactiveResult As New InteractiveResult()
        interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw
        interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays

        rulerLineShape = New LineShape(New Collection(Of Vertex)() From { _
         New Vertex(interactionArguments.WorldX, interactionArguments.WorldY), _
         New Vertex(interactionArguments.WorldX, interactionArguments.WorldY) _
        })

        TrackShapeLayer.InternalFeatures.Add(currentFeatureKey, New Feature(rulerLineShape))

        mouseDown += 1
        Return interactiveResult
    End Function

    Protected Overrides Function MouseMoveCore(interactionArguments As InteractionArguments) As InteractiveResult
        'Updates the line with the last point to where the mouse pointer is being dragged.
        Dim interactiveResult As New InteractiveResult()

        If mouseDown > 0 Then
            Lock.EnterWriteLock()
            rulerLineShape.Vertices(rulerLineShape.Vertices.Count - 1) = New Vertex(interactionArguments.WorldX, interactionArguments.WorldY)
            Lock.ExitWriteLock()

            interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw
            interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays
        End If

        Return interactiveResult
    End Function

    Protected Overrides Function MouseUpCore(interactionArguments As InteractionArguments) As InteractiveResult
        'Removes the line of the ruler at finishing dragging (at mouse up event).
        Dim interactiveResult As New InteractiveResult()

        interactiveResult.DrawThisOverlay = InteractiveOverlayDrawType.Draw
        interactiveResult.ProcessOtherOverlaysMode = ProcessOtherOverlaysMode.DoNotProcessOtherOverlays

        If mouseDown = 1 Then
            mouseDown = 0
            Lock.EnterWriteLock()
            TrackShapeLayer.InternalFeatures.Remove(currentFeatureKey)
            Lock.ExitWriteLock()
        End If
        Return interactiveResult
    End Function

    Protected Overrides Sub DrawCore(canvas As GeoCanvas)
        'Draws the line and update the distance text of the ruler.
        Dim labelingInAllLayers As New Collection(Of SimpleCandidate)()

        Try
            If rulerLineShape IsNot Nothing Then
                Dim feature As New Feature(rulerLineShape)
                Dim length As Double = rulerLineShape.GetLength(GeographyUnit.DecimalDegree, DistanceUnit.Feet)
                feature.ColumnValues.Add("length", CInt(Math.Truncate(length)).ToString() & " feet")

                Lock.EnterWriteLock()
                If True Then
                    If TrackShapeLayer.InternalFeatures.Contains(currentFeatureKey) Then
                        TrackShapeLayer.InternalFeatures(currentFeatureKey) = feature
                    Else
                        TrackShapeLayer.InternalFeatures.Add(currentFeatureKey, feature)
                    End If
                End If
                Lock.ExitWriteLock()
            End If

            TrackShapeLayer.Open()
            TrackShapeLayer.Draw(canvas, labelingInAllLayers)
            canvas.Flush()
        Finally
            TrackShapeLayer.Close()
        End Try
    End Sub
End Class
